import { unstable_createMutableSource, useMemo, unstable_useMutableSource } from 'react';

function create$1(createState) {
  let state;
  const listeners = new Set();
  const setState = (partial, replace) => {
    const nextState = typeof partial === "function" ? partial(state) : partial;
    if (nextState !== state) {
      const previousState = state;
      state = replace ? nextState : Object.assign({}, state, nextState);
      listeners.forEach((listener) => listener(state, previousState));
    }
  };
  const getState = () => state;
  const subscribeWithSelector = (listener, selector = getState, equalityFn = Object.is) => {
    let currentSlice = selector(state);
    function listenerToAdd() {
      const nextSlice = selector(state);
      if (!equalityFn(currentSlice, nextSlice)) {
        const previousSlice = currentSlice;
        listener(currentSlice = nextSlice, previousSlice);
      }
    }
    listeners.add(listenerToAdd);
    return () => listeners.delete(listenerToAdd);
  };
  const subscribe = (listener, selector, equalityFn) => {
    if (selector || equalityFn) {
      return subscribeWithSelector(listener, selector, equalityFn);
    }
    listeners.add(listener);
    return () => listeners.delete(listener);
  };
  const destroy = () => listeners.clear();
  const api = { setState, getState, subscribe, destroy };
  state = createState(setState, getState, api);
  return api;
}

function create(createState) {
  const api = typeof createState === "function" ? create$1(createState) : createState;
  const source = unstable_createMutableSource(api, () => api.getState());
  const subscribe = (api2, callback) => api2.subscribe(callback);
  const FUNCTION_SYMBOL = Symbol();
  const functionMap = new WeakMap();
  const useStore = (selector = api.getState, equalityFn = Object.is) => {
    const getSnapshot = useMemo(() => {
      let lastSlice;
      return (api2) => {
        let slice = lastSlice;
        try {
          slice = selector(api2.getState());
          if (lastSlice !== void 0 && equalityFn(lastSlice, slice)) {
            slice = lastSlice;
          } else {
            lastSlice = slice;
          }
        } catch (error) {
        }
        if (typeof slice === "function") {
          if (functionMap.has(slice)) {
            return functionMap.get(slice);
          }
          const wrappedFunction = { [FUNCTION_SYMBOL]: slice };
          functionMap.set(slice, wrappedFunction);
          return wrappedFunction;
        }
        return slice;
      };
    }, [selector, equalityFn]);
    const snapshot = unstable_useMutableSource(source, getSnapshot, subscribe);
    if (snapshot && snapshot[FUNCTION_SYMBOL]) {
      return snapshot[FUNCTION_SYMBOL];
    }
    return snapshot;
  };
  Object.assign(useStore, api);
  return useStore;
}

export { create as default };
